/*
Function gets matrix containing a mapping from input matrix indices
to output matrix indices. Values are copied from input matrix and
averaged in output matrix accordingly.
Input matrix must have no NA but only 0 for no fluxes/activity!

serialization rule:
- transpose (only) INPUT matrices! 
- rebuild matrix in R using: matrix(result_vector, nrow=ROWS, ncol=COLS, byrow=T)

index formular: i*COLS+j

array assignment expression:
output_array = *(input_array+(i*COLS+j));

R definition:
  dyn.load(paste(scriptpath,"/apply_mapping.so",sep=""))
  apply.mapping <- function(mapping, inmat, outmat)
   .C("apply_mapping",
       as.integer(mapping),
       as.integer(dim(mapping)[2]),
       as.integer(dim(mapping)[1]),
       as.double(inmat),
       as.integer(dim(inmat)[1]),
       as.integer(dim(outmat)[2]),
       as.integer(length(outmat)),
       outmat = double(length(outmat)))$outmat
*/


#include <R.h>
#include <stdio.h>

/*
 Aggregates and projects grid in inmat to output grid outmat
 according to the information given by mapping.

 Mapping has rows of vectors:
 (i coordinate of inmat grid cell, j coordinate of inmat grid cell,
  i coordinate of outmat grid cell, j coordinate of outmat grid cell,
  sum of all cells of inmat which average equals outmat grid cell of current row)
 
 * The sums are needed to calculate the mean in the second loop of the function.
 * In order to account for 0 values in inmat cells for each cell in outmat the
 
 */
void apply_mapping(int    *mapping,
                   int    *mrows,
		   int    *mcols, 
		   double *inmat, 
		   int    *incols, 
		   int    *outcols,
                   int    *length, 
		   double *outmat) {


 int i;
 int i_out, i_in, ind;

 double sum;
 double c;
 double mean;

 double c_mat[*length];
 double sum_mat[*length];

 /*
  initialize c_mat and sum_mat:
  * c_mat contains a counter for each grid cell of the output grid
  * counts the 0 values in inmat contributing to the cell
  * sum_mat contains the number of grid cells of inmat contributing to
    the cell of outmat
  */
 for (i=0;i<*length;i++) {
  c_mat[i]=0.0;
  sum_mat[i]=1.0;
 }

 /*
  * calculate outmat by averaging contributing cells of inmat for each outmat cell
  * count zeros contributing to outmat cell in c_mat
  */
 for (i=0;i<*mrows;i++) {  
  ind   = i**mcols;
  i_out = (*(mapping+ind+2)-1)**outcols+*(mapping+ind+3);
  i_in  = (*(mapping+ind)-1)**incols+*(mapping+ind+1);
  sum   = *(mapping+ind+4);
  
  //is current value counting (not 0)?
  if (*(inmat+i_in)==0.0) {
   c_mat[i_out]++;
  }
  outmat[i_out]  = *(outmat+i_out) + *(inmat+i_in)/sum;
  sum_mat[i_out] = sum;
 }
 
 /*
  * account for zeros in average of each outmat grid cell
  * by scaling each cell of outmat with factor to get correct average
  */
 for (i=0;i<*length;i++) {
  if (c_mat[i]!=sum_mat[i]) {
   outmat[i] = *(outmat+i) * (1.0/(1.0-(c_mat[i]/sum_mat[i])));
  }
 }
 
}
